﻿namespace bo.Text;

interface

uses
  System.IO,
  System.Text;

type
  DefaultToDefaultProcessor = public class
  public
    method Process(inputStream: Stream; outputStream: Stream); virtual;
    method Process(input: StreamReader; output: StreamWriter); virtual;
    method ProcessLine(inputLine: string): string;
    method CreateStreamReader(input: Stream): StreamReader; virtual;
    method CreateStreamWriter(output: Stream): StreamWriter; virtual;
    method GetInputEncoding(): Encoding; virtual;
    method GetOutputEncoding(): Encoding; virtual;
  end;
  
implementation

method DefaultToDefaultProcessor.Process(inputStream: Stream; outputStream: Stream);
begin
  var inputStreamReader := CreateStreamReader(inputStream);
  var outputStreamWriter := CreateStreamWriter(outputStream);
  Process(inputStreamReader, outputStreamWriter);
  outputStreamWriter.Flush();
end;

method DefaultToDefaultProcessor.Process(input: StreamReader; output: StreamWriter);
begin
  var inputLine := input.ReadLine();
  while (not string.IsNullOrEmpty(inputLine)) do
  begin
    var outputLine := ProcessLine(inputLine);
    output.WriteLine(outputLine);
    inputLine := input.ReadLine();
  end;
end;

method DefaultToDefaultProcessor.ProcessLine(inputLine: string): string;
begin
  Result := inputLine;
end;

method DefaultToDefaultProcessor.CreateStreamReader(input: Stream): StreamReader;
begin
  Result := new StreamReader(input, GetInputEncoding());
end;

method DefaultToDefaultProcessor.CreateStreamWriter(output: Stream): StreamWriter;
begin
  Result := new StreamWriter(output, GetOutputEncoding());
end;

method DefaultToDefaultProcessor.GetInputEncoding(): Encoding;
begin
  Result := Encoding.Default;
end;

method DefaultToDefaultProcessor.GetOutputEncoding(): Encoding;
begin
  Result := Encoding.Default;
end;

end.